import es, usermsg, cfglib, gamethread, popuplib, cmdlib, playerlib, services
import os, time, random, path

useSPE = False
useNT = False
usePureES = False

info = es.AddonInfo()
info['name']        = "DeathMatch"
info['version']     = "2.1.4"
info['author']      = "DanielB" 
info['url']         = "http://forums.eventscripts.com/viewtopic.php?t=34141" 
info['description'] = "Fully featured and customisable deathmatch" 
es.ServerVar('danielb_deathmatch_ver', info.version, 'Advanced Deathmatch by DanielB').makepublic()

'''
Check installation/What will be used for respawn
'''

try:
    import spe
    from spe.games import cstrike
    useSPE = True
except ImportError:
    try:
        import nativetools as nt
        useNT = True
    except ImportError:
        usePureES = True
            
            
            
###################################################
## Classes
###################################################
            
            
addondir = str(es.ServerVar('eventscripts_addondir'))

class Settings(object):
    config = cfglib.AddonCFG(addondir + '/dm/settings.cfg')
    
    _admins = config.cvar('dm_admins', 'STEAM_ID_LAN;', 'List of steamids who have access to admin finctions, seperated by a ; [Not used if an auth service is loaded]'), str
    
    _respawntype   = config.cvar('dm_respawntype', 'time', 'How respawn is handled [time/wave/wave2/revive/none]'), str
    
    _spawnprotecttime   = config.cvar('dm_spawnprotecttime', 2.0, 'How long should spawn protection be activated, in seconds? [0 to disable]'), float
    
    _killobjectives = config.cvar('dm_killobjectives', 1, 'Kill Objectives? (bomb, hostages, etc) [1=Yes, 0=No]'), bool
    _enablebuyzone  = config.cvar('dm_enablebuyzone',  1, 'Enable Buyzones?[1=Enabled, 0=Disabled]'), bool
    
    _noblock = config.cvar('dm_noblock',  1, 'Enable Noblock?[1=Enabled, 0=Disabled]'), bool
    
    _balanceteams = config.cvar('dm_balanceteams',  1, 'Enable Team Balancer?[1=Enabled, 0=Disabled]'), bool
    
    config.execute()
    
    def __init__(self):
        self.config.write()
        
    def __getattr__(self, name):
        if hasattr(Settings, name):
            return super(Settings, self).__getattribute__(name)
           
        elif hasattr(Settings, '_'+name):
            val, func = super(Settings, self).__getattribute__('_'+name)
            return func(val)
           
        raise AttributeError, 'No such attribute "%s"'%name
        
    def __setattr__(self, name, value):
        if hasattr(Settings, name):
            super(Settings, self).__setattr__(name, value)
            
        elif hasattr(Settings, '_'+name):
            val, func = super(Settings, self).__getattribute__('_'+name)
            val.set(value)
            
        else:
            raise AttributeError, 'No such attribute "%s"'%name
        
settings = Settings()

class DelayManager:
    def __init__(self):
        self.__call__ = self.delayed
        self.delays = {}
   
    def delayed(self, seconds, name, func, args=(), kw={}):
        if name in self.delays: self.cancel(name)
        self.delays[name] = (func, args, kw)
       
        gamethread.delayedname(seconds, name, self.callback, name)
       
    def callback(self, name):
        func, args, kw = self.delays[name]
        del self.delays[name]
        func(*args, **kw)
   
    def cancel(self, name):
        gamethread.cancelDelayed(name)
        if name in self.delays:
            del self.delays[name]
   
    def cancelPrefix(self, prefix):
        for delayName in self.delays.keys():
            if delayName.startswith(prefix):
                self.cancel(delayName)
   
    def cancelSuffix(self, suffix):
        for delayName in self.delays.keys():
            if delayName.endswith(suffix):
                self.cancel(delayName)
           
    def cancelAll(self):
        for name in self.delays:
            gamethread.cancelDelayed(name)
        self.delays = {}
        
delay = DelayManager()

class ProtectPlayer:
    def __init__(self, userid):
        self.userid = int(userid)
        
        player = playerlib.getPlayer(self.userid)
        player.setColor(255, 0, 0, 180)
        player.godmode = 1
        delay.delayed(settings.spawnprotecttime, 'dm_spawn_protect_%s'%userid, self.off)
        
    def off(self):
        player = playerlib.getPlayer(self.userid)
        player.setColor(255, 255, 255, 255)
        player.godmode = 0
        usermsg.centermsg(self.userid, 'Spawn protection has worn off')
            
###################################################
## Output Wrappers
###################################################

def dbgmsg(text, level=1):
    es.dbgmsg(level, text)
    
def msg(text):
    es.msg('#multi', '#green[DM]#lightgreen ' + text)
    
def tell(userid, text):
    es.tell(userid, '#multi', '#green[DM]#lightgreen ' + text)
    
###################################################
## Load/Unload
###################################################

def load():
    loadAddons()
    setupAuth()
        
    es.flags('add', 'notify', 'dm_respawntype')
    respawn_type = settings.respawntype
    es.set('_dm_respawntype', str(respawn_type))
    
    es.server.queuecmd('es_xload dm/addons/respawntypes/%s'%respawn_type)
    
    using = {useSPE: 'SPE', useNT: 'NT', usePureES: 'PureES'}[True]
    msg('Loaded v%s [%s]'%(info.version, using))
    
def unload():
    delay.cancelAll()
    loadAddons(cmd='es_xunload dm/addons/%s', dbg='Unloaded DM Addon [%s]')
    msg('Unloaded')
    
###################################################
## Events
###################################################

def player_death(ev):

    if settings.balanceteams: # Balance First!
        t = es.getplayercount(2)
        ct = es.getplayercount(3)
        limitteams = int(es.ServerVar('mp_limitteams'))
        difference = abs(ct-t)
        
        if difference > limitteams and es.getplayercount() > 4:
            swapCount = difference/2
            if t > ct:
                p = 2
                to = 3
            else:
                p = 3
                to = 2
                
            players = filter(lambda x: es.getplayerteam(x) == p, es.getUseridList())
            for userid in random.sample(players, swapCount):
                changeTeam(userid, to)
                tell(userid, 'Your have changed teams for auto team balance')
                usermsg.centermsg(userid, 'Your have changed teams for auto team balance')
            
            
def player_spawn(ev):
    noblock(ev['userid'], 1)
    
    if settings.spawnprotecttime:
        ProtectPlayer(ev['userid'])
    
def player_disconnect(ev):
    delay.cancelSuffix('_%s'%ev['userid'])
    
def server_cvar(ev):
    if ev['cvarname'] == 'dm_respawntype':
        old = str(es.ServerVar('_dm_respawntype'))
        new = ev['cvarvalue']
        
        #es.msg('Load %s, unload %s'%(new, old)) 
        es.server.queuecmd('es_xunload dm/addons/respawntypes/%s'%old)
        es.server.queuecmd('es_xload dm/addons/respawntypes/%s'%new)
        
        es.set('_dm_respawntype', new)
        
def round_start(ev):
    #es.server.queuecmd('es_xfire %s game_weapon_manager AddOutput "maxpieces 0"' % es.getuserid())
    
    if settings.killobjectives:
        userid = es.getuserid()
        for ent in ['cs_hostage', 'weapon_c4', 'func_bomb_target']:
            es.server.queuecmd('es_xfire %s %s kill'%(userid, ent))
            
    if settings.enablebuyzone:
        userid = es.getuserid()
        es.server.queuecmd('es_xfire %s func_buyzone Enable'%userid)
    else:
        userid = es.getuserid()
        es.server.queuecmd('es_xfire %s func_buyzone Disable'%userid)

        
def round_end(ev):
    delay.cancelAll()
    
def es_map_start(ev):
    delay.cancelAll()
        

###################################################
## cmdlib/popuplib callbacks
###################################################

###################################################
## Misc Functions
###################################################

def setupAuth():
    global isAuthed
    if services.isRegistered('auth'):
        auth_service = services.use('auth')
        auth_service.registerCapability('dm_admin', auth_service.ADMIN)
        isAuthed = lambda x: auth_service.isUseridAuthorized(x, 'dm_admin')
    else:
        isAuthed = lambda x: es.getplayersteamid(x) in settings.admins.split(';')

def isAlive(userid):
    return not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag')

def timedSpawn(userid):
    if not isAlive(userid):
       respawn(userid)


def loadAddons(cmd='es_xload dm/addons/%s', dbg='Loaded DM Addon [%s]'):
    
    # Check the addons text file exists
    addonsFile = addondir + '/dm/addons.txt'
    if not os.path.isfile(addonsFile):
        dmaddons = []
        dmp = addondir + '/dm/addons/'
        p = path.path(dmp)
        for fdir in p.walkdirs():
            fdir = fdir.replace(dmp, '')
            if not os.sep in fdir:
                if not fdir == 'respawntypes':
                    dmaddons.append(fdir)
            
        f = open(addonsFile, 'w')
        f.write('// Installed Addons\n')
        f.write('// To enable, remove the // from the start of the line of the addon you want to load\n\n')
        for line in dmaddons:
            f.write('//%s\n'%line)
        f.close()
        
        return # If it didn't, no need to go further
    
    for initpath in ['/dm/addons/', '/dm/addons/respawntypes/']:
        initpath = addondir + initpath + '__init__.py'
        if not os.path.isfile(initpath):
            open(initpath, 'w').close()
        
    addonsFile = open(addonsFile, 'r')
    text = addonsFile.readlines()
    addonsFile.close()
    
    for addon in text:
        addon = addon.strip()
        if addon and not addon.startswith('//'):
            es.server.queuecmd(cmd%addon)
            dbgmsg(dbg%addon, 0)
            
    es.server.queuecmd(cmd%'respawntypes/%s'%settings.respawntype)
    
    
def noblock(userid, toggle=1):
    if settings.noblock:
        value = {0:5, 1:2}[toggle]
        es.setplayerprop(userid, 'CBaseEntity.m_CollisionGroup', value)

if useSPE:
    '''
    def respawn(userid):
        cstrike.respawn(int(userid))
    '''
        
    def changeTeam(userid, team, kill=0):
        if kill:
            es.server.queuecmd('es_xchangeteam %s %s'%(userid, team))
        else:
            cstrike.switchTeam(int(userid), team)
            
            
    def registerPreHook(event_name, callback_function):
        spe.registerPreHook(event_name, callback_function)
        
    def unregisterPreHook(event_name, callback_function):
        spe.unregisterPreHook(event_name, callback_function)
            
elif useNT:
    '''
    def respawn(userid):
        nt.spawnPlayer(userid)
    '''
        
    def changeTeam(userid, team, kill=0):
        if kill:
            es.server.queuecmd('es_xchangeteam %s %s'%(userid, team))
        else:
            nt.switchTeam(userid, int(team))
            
    def registerPreHook(event_name, callback_function):
        nt.preEventRegister(event_name, callback_function)
        
    def unregisterPreHook(event_name, callback_function):
        nt.preEventUnregister(event_name, callback_function)
            
elif usePureES:
    def changeTeam(userid, team, kill=0):
        if not kill: # Still causes death, but respawns to same location
            x, y, z = es.getplayerlocation(userid)
            d_args = {'userid': userid, 'team': team, 'x': x, 'y':y, 'z':z}
            
            commands = ['es_xchangeteam %(userid)s %(team)s',
                        'es_xsetplayerprop %(userid)s "CCSPlayer.m_iPlayerState" 0',
                        'es_xsetplayerprop %(userid)s "CCSPlayer.baseclass.m_lifeState" 512',
                        'es_spawnplayer %(userid)s',
                        'es_xsetpos %(userid)s %(x)s %(y)s %(z)s']
        
            for commandstring in commands:
                es.server.queuecmd(commandstring%d_args)
            
        else:
            es.server.queuecmd('es_xchangeteam %s %s'%(userid, team))
            
    def registerPreHook(event_name, callback_function):
        raise ImportError, 'PreHooking is not supported using dm.PureES - use SPE or NT instead'
        
    def unregisterPreHook(event_name, callback_function):
        raise ImportError, 'PreHooking is not supported using dm.PureES - use SPE or NT instead'
            
    
            
            
def respawn(userid):
    es.setplayerprop(userid, "CCSPlayer.m_iPlayerState", 0)
    es.setplayerprop(userid, "CCSPlayer.baseclass.m_lifeState", 512)
    es.spawnplayer(userid)

    
 
def minver(req):
    '''
    Returns True if minvar is met, else False
    '''
    cv = info.version
    
    if cv == req:
        return True
    
    cx, cy, cz = cv.split('.')
    rx, ry, rz = req.split('.')
    cx, cy = int(cx), int(cy)
    rx, ry = int(rx), int(ry)
    
    if rx < cx:
        return True
    elif rx > cx:
        return False
    else:
        if ry < cy:
            return True
        elif ry > cy:
            return False
        else:
            def hasChar(x):
                for i in x:
                    if ord('a') <= ord(i) <= ord('z'):
                        return True
                return False

            cz_hasChar = hasChar(cz)
            rz_hasChar = hasChar(rz)

            if rz_hasChar or cz_hasChar:
                if rz_hasChar:
                    rzn, rzc = int(rz[:-1]), ord(rz[-1:])
                else:
                    rzn, rzc = int(rz), 0
                if cz_hasChar:
                    czn, czc = int(cz[:-1]), ord(cz[-1:])
                else:
                    czn, czc = int(cz), 0

                if rzn < czn:
                    return True
                elif rzn > czn:
                    return False
                else:
                    if rzc < czc:
                        return True
                    return False

            else:
                cz, rz = int(cz), int(rz)
                if rz < cz:
                    return True
                return False
    return False

                



